"use strict";

require("core-js/modules/es.array.iterator");

require("core-js/modules/es.promise");

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

const chalk = require('chalk');

const semver = require('semver');

const latestVersion = require('latest-version');

const settings = require('../../settings');

const pkg = require('../../package');

const ui = require('./ui');

module.exports =
/*#__PURE__*/
function () {
  var _ref = _asyncToGenerator(function* (skip, force) {
    const displayVersionBanner = module.exports.__internal__.displayVersionBanner;

    if (skip) {
      return;
    }

    const now = Date.now();
    const lastCheck = settings.profile_json.last_version_check || 0;

    if (now - lastCheck >= settings.updateCheckInterval || force) {
      settings.profile_json.last_version_check = now;

      try {
        const version = yield getPublishedVersion(pkg, settings);

        if (semver.gt(version, pkg.version)) {
          settings.profile_json.newer_version = version;
        } else {
          delete settings.profile_json.newer_version;
        }

        settings.saveProfileData();

        if (settings.profile_json.newer_version) {
          displayVersionBanner(settings.profile_json.newer_version);
        }
      } catch (error) {
        return;
      }

      return;
    }
  });

  return function (_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

function getPublishedVersion(_x3, _x4) {
  return _getPublishedVersion.apply(this, arguments);
}

function _getPublishedVersion() {
  _getPublishedVersion = _asyncToGenerator(function* (pkgJSON, settings) {
    const latestVersion = module.exports.__internal__.latestVersion;

    try {
      const promise = withTimeout(latestVersion(pkgJSON.name), settings.updateCheckTimeout);
      return yield ui.spin(promise, 'Checking for updates...');
    } catch (error) {
      return pkgJSON.version;
    }
  });
  return _getPublishedVersion.apply(this, arguments);
}

function displayVersionBanner(version) {
  console.error('particle-cli v' + pkg.version);
  console.error();
  console.error(chalk.yellow('!'), 'A newer version (' + chalk.cyan(version) + ') of', chalk.bold.white('particle-cli'), 'is available.');
  console.error(chalk.yellow('!'), 'Upgrade now by running:', chalk.bold.white('particle update-cli'));
  console.error();
}

function withTimeout(promise, ms) {
  const timer = delay(ms).then(() => {
    throw new Error('The operation timed out');
  });
  return Promise.race([promise, timer]);
}

function delay(ms) {
  return new Promise(resolve => setTimeout(resolve, ms));
}

module.exports.__internal__ = {
  latestVersion,
  displayVersionBanner
};