"use strict";

require("core-js/modules/es.promise");

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

const chalk = require('chalk');

const _require = require('./apiclient'),
      buildAPIClient = _require.buildAPIClient;

const _require2 = require('../cmd/api'),
      convertApiError = _require2.convertApiError;

const _require3 = require('../cmd'),
      LibraryInstallCommand = _require3.LibraryInstallCommand,
      LibraryInstallCommandSite = _require3.LibraryInstallCommandSite;

class CLILibraryInstallCommandSite extends LibraryInstallCommandSite {
  constructor(argv, dir, apiClient) {
    super();
    this._apiClient = apiClient;
    this.argv = argv;
    this.dir = dir;
  }

  apiClient() {
    return this._apiClient;
  }

  isVendored() {
    return this.argv.vendored;
  }

  isAdaptersRequired() {
    return this.argv.adapter;
  }

  libraryName() {
    const params = this.argv.params;
    return params && params.name;
  }

  targetDirectory() {
    return this.dir;
  }

  homePathOverride() {
    return this.argv.dest;
  }

  error(error) {
    throw convertApiError(error);
  }

  notifyIncorrectLayout(actualLayout, expectedLayout, libName, targetDir) {
    return _asyncToGenerator(function* () {
      return console.log(`Cannot install library: directory '${targetDir}' is a '${actualLayout}' format project, please change to a '${expectedLayout}' format.`);
    })();
  }

  notifyCheckingLibrary(libName) {
    return _asyncToGenerator(function* () {
      return console.log(`Checking library ${chalk.green(libName)}...`);
    })();
  }

  notifyFetchingLibrary(lib, targetDir) {
    return _asyncToGenerator(function* () {
      const dest = ` to ${targetDir}`;
      return console.log(`Installing library ${chalk.blue(lib.name)} ${lib.version}${dest} ...`);
    })();
  }

  notifyInstalledLibrary(lib) {
    return _asyncToGenerator(function* () {
      return console.log(`Library ${chalk.blue(lib.name)} ${lib.version} installed.`);
    })();
  }

}

function install(argv, apiJS) {
  const site = new CLILibraryInstallCommandSite(argv, process.cwd(), buildAPIClient(apiJS));
  const cmd = new LibraryInstallCommand();
  return site.run(cmd);
}

function copy(argv, apiJS) {
  argv.vendored = true;
  argv.adapter = false;
  argv.confirm = false;
  return install(argv, apiJS);
}

module.exports.CLILibraryInstallCommandSite = CLILibraryInstallCommandSite;

module.exports.command = (cmd, apiJS, argv) => {
  if (cmd === 'copy') {
    return copy(argv, apiJS);
  } else if (cmd === 'install') {
    return install(argv, apiJS);
  } else {
    throw Error('uknown command ' + cmd);
  }
};