"use strict";

require("core-js/modules/es.symbol.description");

require("core-js/modules/es.array.index-of");

require("core-js/modules/es.array.iterator");

require("core-js/modules/es.array.splice");

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

const chalk = require('chalk');

const _require = require('../app/ui'),
      spin = _require.spin;

const prompt = require('../lib/prompts');

const _require2 = require('../cmd/api'),
      convertApiError = _require2.convertApiError;

const _require3 = require('./apiclient'),
      buildAPIClient = _require3.buildAPIClient;

const _require4 = require('./library_ui'),
      formatLibrary = _require4.formatLibrary;

const _require5 = require('../cmd'),
      LibraryListCommand = _require5.LibraryListCommand,
      LibraryListCommandSite = _require5.LibraryListCommandSite;

class CLILibraryListCommandSite extends LibraryListCommandSite {
  constructor(argv, apiClient) {
    super();
    this._apiClient = apiClient;
    this.argv = argv;
    let sections = argv.params.sections;

    if (!sections || !sections.length) {
      sections = ['mine', 'community'];
    }

    this._sectionNames = sections;
    this._page = this.argv.page || 1; // todo - since the text can be used by any app, this could be pushed down to the command layer so it's shared

    this.headings = {
      official: 'Official Libraries',
      verified: 'Verified Libraries',
      popular: 'Popular Libraries',
      mine: 'My Libraries',
      recent: 'Recently added/updated Libraries',
      community: 'Community Libraries'
    };
    this._sections = this._buildSections();
  }

  apiClient() {
    return this._apiClient;
  }

  error(error) {
    throw convertApiError(error);
  }

  target() {
    return {};
  }
  /**
   * Use the default settings
   * @returns {Object} Empty object {}
   */


  settings() {
    const result = {};

    this._add(result, 'filter');

    this._add(result, 'limit');

    return result;
  }

  _add(target, param) {
    if (this.argv[param]) {
      target[param] = this.argv[param];
    }
  }

  sectionNames() {
    return this._sectionNames;
  }

  sections() {
    return this._sections;
  }

  _buildSections() {
    const result = {};
    const sections = this.sectionNames();
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = sections[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        let section = _step.value;
        result[section] = {
          page: 1
        };
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return != null) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    if (result.mine) {
      result.mine.excludeBadges = {
        mine: true
      };
    }

    return result;
  }

  _nextPage() {
    this._page++;
    var _iteratorNormalCompletion2 = true;
    var _didIteratorError2 = false;
    var _iteratorError2 = undefined;

    try {
      for (var _iterator2 = this.sectionNames()[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
        let name = _step2.value;
        const section = this._sections[name];
        section.page += 1;
      }
    } catch (err) {
      _didIteratorError2 = true;
      _iteratorError2 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
          _iterator2.return();
        }
      } finally {
        if (_didIteratorError2) {
          throw _iteratorError2;
        }
      }
    }
  }

  _removeEmptySections(results, sectionNames = this._sectionNames, sections = this._sections) {
    var _iteratorNormalCompletion3 = true;
    var _didIteratorError3 = false;
    var _iteratorError3 = undefined;

    try {
      for (var _iterator3 = sectionNames[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
        let name = _step3.value;
        const section = results[name];

        if (!section || !section.length) {
          this._removeSection(name, sectionNames, sections);
        }
      }
    } catch (err) {
      _didIteratorError3 = true;
      _iteratorError3 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion3 && _iterator3.return != null) {
          _iterator3.return();
        }
      } finally {
        if (_didIteratorError3) {
          throw _iteratorError3;
        }
      }
    }

    return [sectionNames, sections];
  }

  _removeSection(name, sectionNames = this._sectionNames, sections = this._sections) {
    sectionNames.splice(sectionNames.indexOf(name), 1);
    delete sections[name];
  }

  notifyFetchLists(promise) {
    const msg = this._page === 1 ? 'Searching for libraries...' : `Retrieving libraries page ${this._page}`;
    return spin(promise, msg).then(results => {
      const sections = this.sectionNames();
      let separator = false;
      var _iteratorNormalCompletion4 = true;
      var _didIteratorError4 = false;
      var _iteratorError4 = undefined;

      try {
        for (var _iterator4 = sections[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
          let name = _step4.value;
          const list = results[name];

          if (list) {
            if (separator) {
              console.log();
            }

            this.printSection(name, this._sections[name], list);
            separator = true;
          }
        }
      } catch (err) {
        _didIteratorError4 = true;
        _iteratorError4 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion4 && _iterator4.return != null) {
            _iterator4.return();
          }
        } finally {
          if (_didIteratorError4) {
            throw _iteratorError4;
          }
        }
      }

      return results;
    });
  }

  printSection(name, section, libraries) {
    // omit the section if not on the first page and there are no results
    if (!libraries.length && section.page) {
      return;
    }

    const page = section.page > 1 ? chalk.grey(` page ${section.page}`) : '';
    const heading = this.headings[name] || name;
    console.log(chalk.bold(heading) + page);

    if (libraries.length) {
      var _iteratorNormalCompletion5 = true;
      var _didIteratorError5 = false;
      var _iteratorError5 = undefined;

      try {
        for (var _iterator5 = libraries[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
          let library = _step5.value;
          this.showLibrary(section, library);
        }
      } catch (err) {
        _didIteratorError5 = true;
        _iteratorError5 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion5 && _iterator5.return != null) {
            _iterator5.return();
          }
        } finally {
          if (_didIteratorError5) {
            throw _iteratorError5;
          }
        }
      }
    } else {
      console.log(chalk.grey('No libraries to show in this section.'));
    }
  }

  showLibrary(section, library) {
    console.log(formatLibrary(library, section.excludeBadges));
  }

}

module.exports.CLILibraryListCommandSite = CLILibraryListCommandSite;

module.exports.command = (apiJS, argv) => {
  const site = new CLILibraryListCommandSite(argv, buildAPIClient(apiJS));
  const cmd = new LibraryListCommand();

  function prompForNextPage() {
    return prompt.enterToContinueControlCToExit();
  }

  function runPage() {
    return site.run(cmd).then(results => nextPage(results));
  }

  function nextPage(results) {
    if (results) {
      // only continue to show sections with results;
      const _site$_removeEmptySec = site._removeEmptySections(results),
            _site$_removeEmptySec2 = _slicedToArray(_site$_removeEmptySec, 1),
            names = _site$_removeEmptySec2[0];

      if (!argv['non-interactive'] && names.length) {
        site._nextPage();

        return prompForNextPage().then(next => {
          if (next) {
            return runPage();
          }
        });
      }
    }
  }

  return runPage();
};