"use strict";

require("core-js/modules/es.promise");

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

const chalk = require('chalk');

const log = require('../lib/log');

const _require = require('../app/ui'),
      spin = _require.spin;

const _require2 = require('./apiclient'),
      buildAPIClient = _require2.buildAPIClient;

const _require3 = require('../cmd'),
      LibraryContributeCommand = _require3.LibraryContributeCommand;

const _require4 = require('./library_upload'),
      CLILibraryContributeCommandSite = _require4.CLILibraryContributeCommandSite;

const _require5 = require('../cmd'),
      LibraryPublishCommand = _require5.LibraryPublishCommand,
      LibraryPublishCommandSite = _require5.LibraryPublishCommandSite;

class CLILibraryPublishCommandSite extends LibraryPublishCommandSite {
  constructor(argv, apiClient) {
    super();
    this.argv = argv;
    this.ident = argv.params.name;
    this._apiClient = apiClient;
  }

  libraryIdent() {
    return this.ident;
  }

  libraryDirectory() {
    return this.dir;
  }

  apiClient() {
    return this._apiClient;
  }

  error(error) {
    throw error;
  }

  publishingLibrary(promise, ident) {
    return spin(promise, `Publishing library ${chalk.green(ident)}`);
  }

  publishLibraryComplete(library) {
    return log.success(`Library ${chalk.green(library.name)} was successfully published.`);
  }

}

class CLILibraryPublishContributeCommandSite extends CLILibraryContributeCommandSite {
  /**
   * Saves the constributed library and doesn't output a contributed success message since
   * the publish steps comes immediately afterwards - only want to print success when all steps
   * are complete.
   * @param {Library} library   The library that was contributed.
   */
  contributeComplete(library) {
    this.contributedLibrary = library;
  }

}

module.exports.CLILibraryPublishCommandSite = CLILibraryPublishCommandSite;

module.exports.command =
/*#__PURE__*/
function () {
  var _ref = _asyncToGenerator(function* (apiJS, argv) {
    const site = new CLILibraryPublishCommandSite(argv, buildAPIClient(apiJS));
    const cmd = new LibraryPublishCommand();

    if (!site.libraryIdent()) {
      // no library name given - try publishing the current library
      const contributeSite = new CLILibraryPublishContributeCommandSite(argv, process.cwd(), buildAPIClient(apiJS)); // todo - set more stringent validation on the contribute command since this is pre-publish

      const contribute = new LibraryContributeCommand();
      yield contributeSite.run(contribute);
      site.ident = contributeSite.contributedLibrary.name;
    }

    return site.run(cmd);
  });

  return function (_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();