"use strict";

const chalk = require('chalk');

const log = require('../lib/log');

const _require = require('../app/ui'),
      spin = _require.spin;

const _require2 = require('./apiclient'),
      buildAPIClient = _require2.buildAPIClient;

const _require3 = require('./library_ui'),
      formatLibrary = _require3.formatLibrary;

const _require4 = require('../cmd'),
      LibrarySearchCommandSite = _require4.LibrarySearchCommandSite,
      LibrarySearchCommand = _require4.LibrarySearchCommand;

class CLILibrarySearchCommandSite extends LibrarySearchCommandSite {
  constructor(argv, apiClient) {
    super();
    this._apiClient = apiClient;
    this.argv = argv;
  }

  searchString() {
    return this.argv.params.name;
  }

  apiClient() {
    return this._apiClient;
  }

  notifyListLibrariesStart(promise, filter) {
    return spin(promise, `Searching for libraries matching ${chalk.green(filter)}`);
  }

  notifyListLibrariesComplete(promise, filter, libraries, error) {
    if (error) {
      log.error(error);
    } else {
      const count = libraries ? libraries.length : 0;
      const library = count === 1 ? 'library' : 'libraries';
      log.success(`Found ${count} ${library} matching ${chalk.green(filter)}`);

      for (let idx in libraries) {
        const lib = libraries[idx];
        console.log(formatLibrary(lib));
      }
    }
  }

}

module.exports.CLILibrarySearchCommandSite = CLILibrarySearchCommandSite;

module.exports.command = (apiJS, argv) => {
  const site = new CLILibrarySearchCommandSite(argv, buildAPIClient(apiJS));
  const cmd = new LibrarySearchCommand();
  return site.run(cmd);
};