"use strict";

require("core-js/modules/es.symbol.description");

require("core-js/modules/es.array.iterator");

require("core-js/modules/es.promise");

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

const fs = require('fs');

const path = require('path');

const chalk = require('chalk');

const _require = require('./apiclient'),
      buildAPIClient = _require.buildAPIClient;

const _require2 = require('../cmd'),
      LibraryInstallCommand = _require2.LibraryInstallCommand;

const _require3 = require('./library_install'),
      CLILibraryInstallCommandSite = _require3.CLILibraryInstallCommandSite;

class CLILibraryViewCommandSite extends CLILibraryInstallCommandSite {
  notifyFetchingLibrary(lib, targetDir) {
    var _this = this;

    return _asyncToGenerator(function* () {
      const targetExists = fs.existsSync(targetDir);

      if (!_this.targetDir) {
        _this.targetDir = targetDir;
        _this.targetExists = targetExists;
      }

      return !targetExists;
    })();
  }

  notifyInstalledLibrary(lib, targetDir) {
    if (!this.metadata) {
      this.metadata = lib;
    }

    return super.notifyInstalledLibrary(lib, targetDir);
  }

  view() {
    if (this.argv.readme) {
      this.showFile('No readme files found for the library.', ['README.md', 'README.txt']);
    }

    if (this.argv.source) {
      this.showFile('No source files found for the library.', [path.join('src', this.metadata.name + '.cpp')]);
    }

    if (this.argv.header) {
      this.showFile('No header files found for the library.', [path.join('src', this.metadata.name + '.h')]);
    }

    console.log(`To view the library documentation and sources directly, please change to the directory ${chalk.bold(this.targetDir)}`);
  }

  showFile(missing, files) {
    let shown = false;
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = files[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        let file = _step.value;
        const content = this.loadFile(file);

        if (content !== undefined) {
          console.log(content);
          shown = true;
          break;
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return != null) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    if (!shown) {
      console.log(missing);
    }
  }

  loadFile(file) {
    const full = path.join(this.targetDir, file);

    try {
      return fs.readFileSync(full, 'utf-8');
    } catch (error) {
      return undefined;
    }
  }

}

module.exports.command = (apiJS, argv) => {
  const site = new CLILibraryViewCommandSite(argv, process.cwd(), buildAPIClient(apiJS));
  const cmd = new LibraryInstallCommand();
  return site.run(cmd).then(() => site.view());
};